#include <dht11.h>

#define SENDBUFF 16
#define COMDELAY 5

const int DHT11PIN = A0;
const int lightPIN = A1;
const int LED = 13;

dht11 DHT11;

byte bytesToPHP[SENDBUFF];   // linked with SensorData

// 1 + 1 + 3*2 + 2*4 = 16 bytes
class SensorData
{
    public:
    
    byte tag;
    byte length;
    
    int tempe;
    int humid;
    int light;
    
    float dewPoint;
    unsigned long count;
    
    SensorData()
    {}
    
    void SendSensor(byte *base)
    {
        tag = 'S';
        length = 16;
        
        Serial.write(base, SENDBUFF);
        Serial.flush();
        delay(COMDELAY);
    }
    
    void ReadFail(byte value)
    {
        byte t = 'F';
        byte l = 1;
        byte v = value;
        
        Serial.write(t);
        Serial.write(l);
        Serial.write(v);
        Serial.flush();
        delay(COMDELAY);
    }
    
    
}__attribute__((packed));  // minimum data structure alignment



// ¶㣨ڴ¶ʱͲ¶飩
// ο: http://wahiduddin.net/calc/density_algorithms.htm 
float dewPoint(double celsius, double humidity)
{
        double A0= 373.15/(273.15 + celsius);
        double SUM = -7.90298 * (A0-1);
        SUM += 5.02808 * log10(A0);
        SUM += -1.3816e-7 * (pow(10, (11.344*(1-1/A0)))-1) ;
        SUM += 8.1328e-3 * (pow(10,(-3.49149*(A0-1)))-1) ;
        SUM += log10(1013.246);
        double VP = pow(10, SUM-3) * humidity;
        double T = log(VP/0.61078);   // temp var
        T = (241.88 * T) / (17.558-T);
        return (float)T;
}


void blinkLED()
{
  digitalWrite(LED, HIGH);
  delay(200);
  digitalWrite(LED, LOW);
}

void sReset()
{
    asm volatile ("  jmp 0");  
}


class SensorData *pSD;   //set class pointers for SensorData


void setup(void)
{
    pSD = (class SensorData *)bytesToPHP;
    
    pinMode(LED, OUTPUT);
    pinMode(DHT11PIN, INPUT);
    pinMode(lightPIN, INPUT);
    Serial.begin(9600);
}

void loop(void)
{   
    int chk = DHT11.read(DHT11PIN);
    
    switch (chk)
    {
      case DHTLIB_OK: 
                break;
      default: 
                pSD->ReadFail(5);
                sReset();
                break;
    }
    
    pSD->tempe = DHT11.temperature;
    pSD->humid = DHT11.humidity;
    pSD->light = analogRead(lightPIN);
    pSD->dewPoint = dewPoint(DHT11.temperature, DHT11.humidity);
    
    pSD->SendSensor(bytesToPHP);   // send all sensor data
    
    pSD->count++;
    
    blinkLED();
    delay(1000);
}

